/*
 * This code is part of the strategy game operational-space.
 * operational-space comes with ABSOLUTELY NO WARRANTY and is licensed under GPL-2.0.
 *    Copyright (C) 2024  VegOwOtenks, Sleppo04
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */
#ifndef COMMONALLOCATORINTERFACE_H
#define COMMONALLOCATORINTERFACE_H

#include <stddef.h>
#include <stdlib.h>
#include <string.h>

#include "../errorcodes.h"

typedef struct Allocator_s allocator_t;

typedef void* (*AllocatorAllocateArrayFunction) (allocator_t* allocator, size_t object_size, size_t length);
typedef void* (*AllocatorReallocateFunction)    (allocator_t* allocator, void* pointer, size_t old_size, size_t new_size);
typedef int   (*AllocatorFreeArrayFunction)     (allocator_t* allocator, void* pointer, size_t object_size, size_t length);
typedef void* (*AllocatorAllocateFunction)      (allocator_t* allocator, size_t size);
typedef int   (*AllocatorFreeFunction)          (allocator_t* allocator, void* pointer, size_t size);

struct Allocator_s {
    AllocatorAllocateArrayFunction  allocateArray;
    AllocatorReallocateFunction     reallocate;
    AllocatorFreeArrayFunction      freeArray;
    AllocatorAllocateFunction       allocate;
    AllocatorFreeFunction           free;

    size_t                          reserved;

    void*                           xdata;
};

void* Allocator_Allocate(allocator_t* allocator, size_t size);
int   Allocator_Free(allocator_t* allocator, void* pointer, size_t size);
void* Allocator_AllocateArray(allocator_t* allocator, size_t object_size, size_t length);
int Allocator_FreeArray(allocator_t* allocator, void* pointer, size_t object_size, size_t length);
void* Allocator_Reallocate(allocator_t* allocator, void* pointer, size_t old_size, size_t new_size);

int Allocator_CreateSystemAllocator(allocator_t* destination);
void Allocator_DestroySystemAllocator(allocator_t* allocator);

void* Allocator_ForwardReallocate(allocator_t* allocator, void* pointer, size_t old_size, size_t new_size);


#endif
