#ifndef UTILITIEC_REGEX_MATCH_H
#define UTILITIEC_REGEX_MATCH_H

#include "machine_state.h"
#include "regex_struct.h"
#include "match_struct.h"

enum RegexMatchThreadType {
	REGEXMATCHTHREADTYPE_THREAD,
	REGEXMATCHTHREADTYPE_GROUP,
};

typedef struct RegexMatchThreadGroup_s {
	enum RegexMatchThreadType type;
	
	// thread-group number
	// possibly matches with one of the threads' numbers
	size_t number;

	DynamicArray threads;
	DynamicArray subgroups;
} RegexMatchThreadGroup;

typedef struct RegexMatchThread_s {
	enum RegexMatchThreadType type;

	// Identifier for the thread
	// if it matches the threadgroups, the thread is it's leader 
	size_t number;

	// Full test string
	StringView string;

	// left/right from the text cursor, not necessarily part of the match
	StringView right;
	StringView left;

	// head node next up for evaluation
	RegexMachineStateBase* machine_head;
	
	// stack of repeat-node states
	DynamicArray repeats;

	// Immutable Reference to regex, for options
	const Regex* regex;

	// Match state until now
	RegexMatch match;
} RegexMatchThread;

typedef struct RegexRepeatStackEntry_s {
	// node this stack entry is for
	RegexMachineStateRepeat* node;
	// the current iteration
	size_t current;
} RegexRepeatStackEntry;


int Regex_MatchHere(Regex* regex, StringView string, size_t start, RegexMatch* match);
RegexCapture* RegexMatch_MakeCapture(RegexMatchThread* thread, char* name, size_t number);

#endif
